@@ -108,9 +108,7 @@ impl WebResponse {
     pub fn take_body(&mut self) -> ResponseBody<Body> {
         self.response.take_body()
     }
-}
 
-impl WebResponse {
     /// Set a new body
     pub fn map_body<F>(self, f: F) -> WebResponse
     where
@@ -126,7 +124,11 @@ impl WebResponse {
 }
 
 impl From<WebResponse> for Response<Body> {
-    fn from(res: WebResponse) -> Response<Body> {
+    fn from(mut res: WebResponse) -> Response<Body> {
+        let head = res.response.head_mut();
+        if head.upgrade() {
+            head.set_io(res.request.head());
+        }
         res.response
     }
 }
@@ -166,9 +168,8 @@ mod tests {
         assert_eq!(res.response().status(), StatusCode::PAYLOAD_TOO_LARGE);
 
         let res = TestRequest::default().to_srv_response(HttpResponse::Ok().finish());
-        let mut res = res.checked_expr::<DefaultError, _, _>(|_| {
-            Ok::<_, http::error::PayloadError>(())
-        });
+        let mut res = res
+            .checked_expr::<DefaultError, _, _>(|_| Ok::<_, http::error::PayloadError>(()));
         assert_eq!(res.response_mut().status(), StatusCode::OK);
         let res = res.checked_expr::<DefaultError, _, _>(|_| {
             Err(http::error::PayloadError::Overflow)
