@@ -3,6 +3,7 @@ use std::{future::Future, rc::Rc};
 use crate::io::{IoRef, OnDisconnect};
 use crate::ws;
 
+#[derive(Clone)]
 pub struct WsSink(Rc<WsSinkInner>);
 
 struct WsSinkInner {
@@ -15,6 +16,11 @@ impl WsSink {
         Self(Rc::new(WsSinkInner { io, codec }))
     }
 
+    /// Io reference
+    pub fn io(&self) -> &IoRef {
+        &self.0.io
+    }
+
     /// Endcode and send message to the peer.
     pub fn send(
         &self,
